@javax.xml.bind.annotation.XmlSchema(namespace = "http://jboss.org/resteasy/test/providers/resource/jaxbNameSpacePrefix", elementFormDefault = javax.xml.bind.annotation.XmlNsForm.QUALIFIED)
package org.jboss.resteasy.test.providers.resource.jaxbNameSpacePrefix;
