package org.jboss.resteasy.test.spring.inmodule.resource;

/**
 * Created with IntelliJ IDEA.
 * User: sgv
 * Date: 24/10/14
 * Time: 8:45
 */
public class RequestScopedBeanInnerBeanImpl implements RequestScopedBeanInnerBean {
}
