//
// This file was generated by the JavaTM Architecture for XML Binding(JAXB) Reference Implementation, v2.2.8-b130911.1802 
// See <a href="http://java.sun.com/xml/jaxb">http://java.sun.com/xml/jaxb</a> 
// Any modifications to this file will be lost upon recompilation of the source schema. 
// Generated on: 2015.06.18 at 10:10:02 PM CST 
//


package org.jboss.resteasy.wadl.jaxb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAnyAttribute;
import javax.xml.bind.annotation.XmlAnyElement;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElements;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlSchemaType;
import javax.xml.bind.annotation.XmlType;
import javax.xml.bind.annotation.adapters.CollapsedStringAdapter;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
import javax.xml.namespace.QName;
import org.w3c.dom.Element;


/**
 * <p>Java class for anonymous complex type.
 * 
 * <p>The following schema fragment specifies the expected content contained within this class.
 * 
 * <pre>
 * &lt;complexType&gt;
 *   &lt;complexContent&gt;
 *     &lt;restriction base="{http://www.w3.org/2001/XMLSchema}anyType"&gt;
 *       &lt;sequence&gt;
 *         &lt;element ref="{http://wadl.dev.java.net/2009/02}doc" maxOccurs="unbounded" minOccurs="0"/&gt;
 *         &lt;element ref="{http://wadl.dev.java.net/2009/02}param" maxOccurs="unbounded" minOccurs="0"/&gt;
 *         &lt;choice maxOccurs="unbounded" minOccurs="0"&gt;
 *           &lt;element ref="{http://wadl.dev.java.net/2009/02}method"/&gt;
 *           &lt;element ref="{http://wadl.dev.java.net/2009/02}resource"/&gt;
 *         &lt;/choice&gt;
 *         &lt;any processContents='lax' namespace='##other' maxOccurs="unbounded" minOccurs="0"/&gt;
 *       &lt;/sequence&gt;
 *       &lt;attribute name="id" type="{http://www.w3.org/2001/XMLSchema}ID" /&gt;
 *       &lt;attribute name="type" type="{http://wadl.dev.java.net/2009/02}resource_type_list" /&gt;
 *       &lt;attribute name="queryType" type="{http://www.w3.org/2001/XMLSchema}string" default="application/x-www-form-urlencoded" /&gt;
 *       &lt;attribute name="path" type="{http://www.w3.org/2001/XMLSchema}string" /&gt;
 *       &lt;anyAttribute processContents='lax' namespace='##other'/&gt;
 *     &lt;/restriction&gt;
 *   &lt;/complexContent&gt;
 * &lt;/complexType&gt;
 * </pre>
 * 
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "", propOrder = {
   "doc",
   "param",
   "methodOrResource",
   "any"
})
@XmlRootElement(name = "resource")
public class Resource {

   protected List<Doc> doc;
   protected List<Param> param;
   @XmlElements({
      @XmlElement(name = "method", type = Method.class),
      @XmlElement(name = "resource", type = Resource.class)
   })
   protected List<Object> methodOrResource;
   @XmlAnyElement(lax = true)
   protected List<Object> any;
   @XmlAttribute(name = "id")
   @XmlJavaTypeAdapter(CollapsedStringAdapter.class)
   @XmlID
   @XmlSchemaType(name = "ID")
   protected String id;
   @XmlAttribute(name = "type")
   protected List<String> type;
   @XmlAttribute(name = "queryType")
   protected String queryType;
   @XmlAttribute(name = "path")
   protected String path;
   @XmlAnyAttribute
   private Map<QName, String> otherAttributes = new HashMap<QName, String>();

   /**
    * Gets the value of the doc property.
    *
    * <p>
    * This accessor method returns a reference to the live list,
    * not a snapshot. Therefore any modification you make to the
    * returned list will be present inside the JAXB object.
    * This is why there is not a <CODE>set</CODE> method for the doc property.
    *
    * <p>
    * For example, to add a new item, do as follows:
    * <pre>
    *    getDoc().add(newItem);
    * </pre>
    *
    *
    * <p>
    * Objects of the following type(s) are allowed in the list
    * {@link Doc }
    *
    *
    */
   public List<Doc> getDoc() {
      if (doc == null) {
         doc = new ArrayList<Doc>();
      }
      return this.doc;
   }

   /**
    * Gets the value of the param property.
    *
    * <p>
    * This accessor method returns a reference to the live list,
    * not a snapshot. Therefore any modification you make to the
    * returned list will be present inside the JAXB object.
    * This is why there is not a <CODE>set</CODE> method for the param property.
    *
    * <p>
    * For example, to add a new item, do as follows:
    * <pre>
    *    getParam().add(newItem);
    * </pre>
    *
    *
    * <p>
    * Objects of the following type(s) are allowed in the list
    * {@link Param }
    *
    *
    */
   public List<Param> getParam() {
      if (param == null) {
         param = new ArrayList<Param>();
      }
      return this.param;
   }

   /**
    * Gets the value of the methodOrResource property.
    *
    * <p>
    * This accessor method returns a reference to the live list,
    * not a snapshot. Therefore any modification you make to the
    * returned list will be present inside the JAXB object.
    * This is why there is not a <CODE>set</CODE> method for the methodOrResource property.
    *
    * <p>
    * For example, to add a new item, do as follows:
    * <pre>
    *    getMethodOrResource().add(newItem);
    * </pre>
    *
    *
    * <p>
    * Objects of the following type(s) are allowed in the list
    * {@link Method }
    * {@link Resource }
    *
    *
    */
   public List<Object> getMethodOrResource() {
      if (methodOrResource == null) {
         methodOrResource = new ArrayList<Object>();
      }
      return this.methodOrResource;
   }

   /**
    * Gets the value of the any property.
    *
    * <p>
    * This accessor method returns a reference to the live list,
    * not a snapshot. Therefore any modification you make to the
    * returned list will be present inside the JAXB object.
    * This is why there is not a <CODE>set</CODE> method for the any property.
    *
    * <p>
    * For example, to add a new item, do as follows:
    * <pre>
    *    getAny().add(newItem);
    * </pre>
    *
    *
    * <p>
    * Objects of the following type(s) are allowed in the list
    * {@link Element }
    * {@link Object }
    *
    *
    */
   public List<Object> getAny() {
      if (any == null) {
         any = new ArrayList<Object>();
      }
      return this.any;
   }

   /**
    * Gets the value of the id property.
    *
    * @return
    *     possible object is
    *     {@link String }
    *
    */
   public String getId() {
      return id;
   }

   /**
    * Sets the value of the id property.
    *
    * @param value
    *     allowed object is
    *     {@link String }
    *
    */
   public void setId(String value) {
      this.id = value;
   }

   /**
    * Gets the value of the type property.
    *
    * <p>
    * This accessor method returns a reference to the live list,
    * not a snapshot. Therefore any modification you make to the
    * returned list will be present inside the JAXB object.
    * This is why there is not a <CODE>set</CODE> method for the type property.
    *
    * <p>
    * For example, to add a new item, do as follows:
    * <pre>
    *    getType().add(newItem);
    * </pre>
    *
    *
    * <p>
    * Objects of the following type(s) are allowed in the list
    * {@link String }
    *
    *
    */
   public List<String> getType() {
      if (type == null) {
         type = new ArrayList<String>();
      }
      return this.type;
   }

   /**
    * Gets the value of the queryType property.
    *
    * @return
    *     possible object is
    *     {@link String }
    *
    */
   public String getQueryType() {
      if (queryType == null) {
         return "application/x-www-form-urlencoded";
      } else {
         return queryType;
      }
   }

   /**
    * Sets the value of the queryType property.
    *
    * @param value
    *     allowed object is
    *     {@link String }
    *
    */
   public void setQueryType(String value) {
      this.queryType = value;
   }

   /**
    * Gets the value of the path property.
    *
    * @return
    *     possible object is
    *     {@link String }
    *
    */
   public String getPath() {
      return path;
   }

   /**
    * Sets the value of the path property.
    *
    * @param value
    *     allowed object is
    *     {@link String }
    *
    */
   public void setPath(String value) {
      this.path = value;
   }

   /**
    * Gets a map that contains attributes that aren't bound to any typed property on this class.
    *
    * <p>
    * the map is keyed by the name of the attribute and
    * the value is the string value of the attribute.
    *
    * the map returned by this method is live, and you can add new attribute
    * by updating the map directly. Because of this design, there's no setter.
    *
    *
    * @return
    *     always non-null
    */
   public Map<QName, String> getOtherAttributes() {
      return otherAttributes;
   }

}
