package org.jboss.resteasy.api.validation;

/**
 * 
 * @author <a href="ron.sigal@jboss.com">Ron Sigal</a>
 * @version $Revision: 1.1 $
 *
 * Copyright May 11, 2013
 */
public class Validation
{
   public static final String VALIDATION_HEADER = "validation-exception";
}
