package org.jboss.resteasy.test.injection.resource;

public class StringParameterInjectorInjected {

   public final String value;

   public StringParameterInjectorInjected(final String value) {
      this.value = value;
   }

}
