package org.jboss.resteasy.test.cdi.injection.resource;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "application_user")
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class ApplicationUser implements Serializable {

   /**
    * The ID property for the entity
    */
   @Id
   private Long id;

   @Column(name = "user_type")
   @Enumerated(EnumType.STRING)
   private UserType userType;

   public Long getId() {
      return id;
   }

   public void setId(Long id) {
      this.id = id;
   }

   public UserType getUserType() {
      return userType;
   }

   public void setUserType(UserType userType) {
      this.userType = userType;
   }

}
