package org.jboss.resteasy.test.interceptor.resource;

import java.io.IOException;

import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerResponseContext;
import javax.ws.rs.container.ContainerResponseFilter;
import javax.ws.rs.ext.Provider;

@Provider
public class AddHeaderContainerResponseFilter implements ContainerResponseFilter
{
   @Override
   public void filter(ContainerRequestContext requestContext, ContainerResponseContext responseContext) throws IOException
   {
      responseContext.getHeaders().add("custom-header", "hello");
   }
}
