package org.jboss.resteasy.test.providers.multipart.resource;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class InputPartDefaultContentTypeWildcardOverwriteXmlBean {
   private int myInt;
   private String myString;

   public int getMyInt() {
      return myInt;
   }

   public void setMyInt(int myInt) {
      this.myInt = myInt;
   }

   public String getMyString() {
      return myString;
   }

   public void setMyString(String myString) {
      this.myString = myString;
   }
}
