package org.jboss.resteasy.jsapi.testing.form;

import org.jboss.resteasy.annotations.Form;

import java.util.List;
import java.util.Map;

/**
 * 12 05 2012
 *
 * @author <a href="mailto:l.weinan@gmail.com">Weinan Li</a>
 */
public class Person {
   @Form(prefix="telephoneNumbers") List<TelephoneNumber> telephoneNumbers;
   @Form(prefix="address")
   Map<String, Address> addresses;

   public List<TelephoneNumber> getTelephoneNumbers() {
      return telephoneNumbers;
   }

   public void setTelephoneNumbers(List<TelephoneNumber> telephoneNumbers) {
      this.telephoneNumbers = telephoneNumbers;
   }

   public Map<String, Address> getAddresses() {
      return addresses;
   }

   public void setAddresses(Map<String, Address> addresses) {
      this.addresses = addresses;
   }
}
