package org.jboss.resteasy.test.providers.jaxb.resource;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;


/**
 * <p>Java class for shiptotype complex type.
 * <p>
 * <p>The following schema fragment specifies the expected content contained within this class.
 * <p>
 * <pre>
 * &lt;complexType name="shiptotype">
 *   &lt;complexContent>
 *     &lt;restriction base="{http://www.w3.org/2001/XMLSchema}anyType">
 *       &lt;sequence>
 *         &lt;element name="name" type="{http://jboss.org/resteasy/test/providers/jaxb/generated/order}stringtype"/>
 *         &lt;element name="address" type="{http://jboss.org/resteasy/test/providers/jaxb/generated/order}stringtype"/>
 *         &lt;element name="city" type="{http://jboss.org/resteasy/test/providers/jaxb/generated/order}stringtype"/>
 *         &lt;element name="country" type="{http://jboss.org/resteasy/test/providers/jaxb/generated/order}stringtype"/>
 *       &lt;/sequence>
 *     &lt;/restriction>
 *   &lt;/complexContent>
 * &lt;/complexType>
 * </pre>
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "shiptotype", propOrder = {
      "name",
      "address",
      "city",
      "country"
})
public class Shiptotype {

   @XmlElement(required = true)
   protected String name;
   @XmlElement(required = true)
   protected String address;
   @XmlElement(required = true)
   protected String city;
   @XmlElement(required = true)
   protected String country;

   /**
    * Gets the value of the name property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getName() {
      return name;
   }

   /**
    * Sets the value of the name property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setName(String value) {
      this.name = value;
   }

   /**
    * Gets the value of the address property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getAddress() {
      return address;
   }

   /**
    * Sets the value of the address property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setAddress(String value) {
      this.address = value;
   }

   /**
    * Gets the value of the city property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getCity() {
      return city;
   }

   /**
    * Sets the value of the city property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setCity(String value) {
      this.city = value;
   }

   /**
    * Gets the value of the country property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getCountry() {
      return country;
   }

   /**
    * Sets the value of the country property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setCountry(String value) {
      this.country = value;
   }

}
