package org.jboss.resteasy.test.providers.resource.jaxbNameSpacePrefix;

import java.math.BigDecimal;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlSchemaType;
import javax.xml.bind.annotation.XmlType;
import javax.xml.bind.annotation.adapters.CollapsedStringAdapter;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;


/**
 * <p>Java class for JaxbNameSpacePrefixUSAddress complex type.
 * <p>
 * <p>The following schema fragment specifies the expected content contained within this class.
 * <p>
 * <pre>
 * &lt;complexType name="JaxbNameSpacePrefixUSAddress">
 *   &lt;complexContent>
 *     &lt;restriction base="{http://www.w3.org/2001/XMLSchema}anyType">
 *       &lt;sequence>
 *         &lt;element name="name" type="{http://www.w3.org/2001/XMLSchema}string"/>
 *         &lt;element name="street" type="{http://www.w3.org/2001/XMLSchema}string"/>
 *         &lt;element name="city" type="{http://www.w3.org/2001/XMLSchema}string"/>
 *         &lt;element name="state" type="{http://www.w3.org/2001/XMLSchema}string"/>
 *         &lt;element name="zip" type="{http://www.w3.org/2001/XMLSchema}decimal"/>
 *       &lt;/sequence>
 *       &lt;attribute name="country" type="{http://www.w3.org/2001/XMLSchema}NMTOKEN" fixed="US" />
 *     &lt;/restriction>
 *   &lt;/complexContent>
 * &lt;/complexType>
 * </pre>
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "JaxbNameSpacePrefixUSAddress", propOrder = {
      "name",
      "street",
      "city",
      "state",
      "zip"
})
public class JaxbNameSpacePrefixUSAddress {

   @XmlElement(required = true)
   protected String name;
   @XmlElement(required = true)
   protected String street;
   @XmlElement(required = true)
   protected String city;
   @XmlElement(required = true)
   protected String state;
   @XmlElement(required = true)
   protected BigDecimal zip;
   @XmlAttribute
   @XmlJavaTypeAdapter(CollapsedStringAdapter.class)
   @XmlSchemaType(name = "NMTOKEN")
   protected String country;

   /**
    * Gets the value of the name property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getName() {
      return name;
   }

   /**
    * Sets the value of the name property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setName(String value) {
      this.name = value;
   }

   /**
    * Gets the value of the street property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getStreet() {
      return street;
   }

   /**
    * Sets the value of the street property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setStreet(String value) {
      this.street = value;
   }

   /**
    * Gets the value of the city property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getCity() {
      return city;
   }

   /**
    * Sets the value of the city property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setCity(String value) {
      this.city = value;
   }

   /**
    * Gets the value of the state property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getState() {
      return state;
   }

   /**
    * Sets the value of the state property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setState(String value) {
      this.state = value;
   }

   /**
    * Gets the value of the zip property.
    *
    * @return possible object is
    * {@link BigDecimal }
    */
   public BigDecimal getZip() {
      return zip;
   }

   /**
    * Sets the value of the zip property.
    *
    * @param value allowed object is
    *              {@link BigDecimal }
    */
   public void setZip(BigDecimal value) {
      this.zip = value;
   }

   /**
    * Gets the value of the country property.
    *
    * @return possible object is
    * {@link String }
    */
   public String getCountry() {
      if (country == null) {
         return "US";
      } else {
         return country;
      }
   }

   /**
    * Sets the value of the country property.
    *
    * @param value allowed object is
    *              {@link String }
    */
   public void setCountry(String value) {
      this.country = value;
   }

}
