package org.jboss.resteasy.category;

/**
 * Marker interface for test which are not used for forward compatibility testing.
 * Use EAP 7.0.0.GA and newer Client from CP stream (e.g. client from RESTEasy delivered by EAP 7.0.2
 */
public interface NotForForwardCompatibility {
}
