package org.jboss.resteasy.test.resource.param.resource;

import org.jboss.resteasy.test.resource.param.QueryParamAsPrimitiveTest;
import org.junit.Assert;

import javax.ws.rs.DefaultValue;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

@Path("/array/default")
public class QueryParamAsPrimitiveResourceArrayDefault {
   @GET
   @Produces("application/boolean")
   public String doGetBoolean(@QueryParam("boolean") @DefaultValue("true") boolean[] v) {
      Assert.assertEquals(QueryParamAsPrimitiveTest.ERROR_MESSAGE, true, v[0]);
      return "content";
   }

   @GET
   @Produces("application/byte")
   public String doGetByte(@QueryParam("byte") @DefaultValue("127") byte[] v) {
      Assert.assertTrue((byte) 127 == v[0]);
      return "content";
   }

   @GET
   @Produces("application/short")
   public String doGetShort(@QueryParam("short") @DefaultValue("32767") short[] v) {
      Assert.assertTrue((short) 32767 == v[0]);
      return "content";
   }

   @GET
   @Produces("application/int")
   public String doGetInteger(@QueryParam("int") @DefaultValue("2147483647") int[] v) {
      Assert.assertEquals(QueryParamAsPrimitiveTest.ERROR_MESSAGE, 2147483647, v[0]);
      return "content";
   }

   @GET
   @Produces("application/long")
   public String doGetLong(@QueryParam("long") @DefaultValue("9223372036854775807") long[] v) {
      Assert.assertEquals(QueryParamAsPrimitiveTest.ERROR_MESSAGE, 9223372036854775807L, v[0]);
      return "content";
   }

   @GET
   @Produces("application/float")
   public String doGetFloat(@QueryParam("float") @DefaultValue("3.14159265") float[] v) {
      Assert.assertEquals(QueryParamAsPrimitiveTest.ERROR_MESSAGE, 3.14159265f, v[0], 0.0f);
      return "content";
   }

   @GET
   @Produces("application/double")
   public String doGetDouble(@QueryParam("double") @DefaultValue("3.14159265358979") double[] v) {
      Assert.assertEquals(QueryParamAsPrimitiveTest.ERROR_MESSAGE, 3.14159265358979d, v[0], 0.0);
      return "content";
   }
    
   @GET
   @Produces("application/char")
   public String doGetCharacter(@QueryParam("char") @DefaultValue("a") char[] v) {
      Assert.assertEquals(QueryParamAsPrimitiveTest.ERROR_MESSAGE, 'a', v[0]);
      return "content";
   }
}
