package org.jboss.resteasy.test.providers.jaxb.resource.parsing;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.bind.annotation.XmlType;

@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "ParsingAbstractData")
@XmlSeeAlso(ParsingDataCollectionPackage.class)
public class ParsingAbstractData {
}
