/*
 * JBoss, Home of Professional Open Source
 * Copyright 2016, Red Hat, Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.resteasy.test.client;

import java.net.URI;

import org.jboss.arquillian.test.api.ArquillianResource;

/**
 * @author Tomaz Cerar (c) 2016 Red Hat Inc.
 */
public abstract class ClientTestBase {

   @ArquillianResource
   URI baseUri;

   protected String generateURL(String path) {
      if (path.startsWith("/")){
         path = path.substring(1);
      }
      return baseUri.resolve(path).toString();
   }
}
