package org.jboss.resteasy.test.client.resource;

import javax.annotation.Priority;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.ext.ReaderInterceptor;
import javax.ws.rs.ext.ReaderInterceptorContext;
import java.io.IOException;

@Priority(100)
public class ClientResponseFilterInterceptorReaderOne implements ReaderInterceptor {
   @Override
   public Object aroundReadFrom(ReaderInterceptorContext context) throws IOException, WebApplicationException {
      try {
         return context.proceed();
      } catch (IOException e) {
         return "OK";
      }
   }
}
