package org.jboss.resteasy.test.spring.deployment.resource;

public class RequestScopedBeanTestBean {
   String configured;

   public void setConfigured(String configured) {
      this.configured = configured;
   }
}
