//
// This file was generated by the JavaTM Architecture for XML Binding(JAXB) Reference Implementation, v2.2.8-b130911.1802 
// See <a href="http://java.sun.com/xml/jaxb">http://java.sun.com/xml/jaxb</a> 
// Any modifications to this file will be lost upon recompilation of the source schema. 
// Generated on: 2015.06.18 at 10:10:02 PM CST 
//


package org.jboss.resteasy.wadl.jaxb;

import javax.xml.bind.annotation.XmlEnum;
import javax.xml.bind.annotation.XmlEnumValue;
import javax.xml.bind.annotation.XmlType;


/**
 * <p>Java class for ParamStyle.
 * 
 * <p>The following schema fragment specifies the expected content contained within this class.
 * <p>
 * <pre>
 * &lt;simpleType name="ParamStyle"&gt;
 *   &lt;restriction base="{http://www.w3.org/2001/XMLSchema}string"&gt;
 *     &lt;enumeration value="plain"/&gt;
 *     &lt;enumeration value="query"/&gt;
 *     &lt;enumeration value="matrix"/&gt;
 *     &lt;enumeration value="header"/&gt;
 *     &lt;enumeration value="template"/&gt;
 *   &lt;/restriction&gt;
 * &lt;/simpleType&gt;
 * </pre>
 * 
 */
@XmlType(name = "ParamStyle")
@XmlEnum
public enum ParamStyle {

   @XmlEnumValue("plain")
   PLAIN("plain"),
   @XmlEnumValue("query")
   QUERY("query"),
   @XmlEnumValue("matrix")
   MATRIX("matrix"),
   @XmlEnumValue("header")
   HEADER("header"),
   @XmlEnumValue("template")
   TEMPLATE("template");
   private final String value;

   ParamStyle(String v) {
      value = v;
   }

   public String value() {
      return value;
   }

   public static ParamStyle fromValue(String v) {
      for (ParamStyle c: ParamStyle.values()) {
         if (c.value.equals(v)) {
            return c;
         }
      }
      throw new IllegalArgumentException(v);
   }

}
